@@ -1,23 +1,18 @@
-G.matrix <- function(M, method=c("VanRaden", "UAR", "UARadj", "GK"), format=c("wide", "long")){
+G.matrix <- function(M, method=c("VanRaden", "UAR", "UARadj", "GK"), format=c("wide", "long"),
+                     plot = FALSE){
   
   if (any(is.na(M)))
     stop("Matrix should not have missing values")
   
-  match.arg(method)
-  match.arg(format)
-  
   if(missing(method))
     stop("Method argument is missing")
      
-  if(missing(format))
-    stop("Format argument is missing")
-  
   N <- nrow(M)
   m <- ncol(M)
-  p <- colSums(M)/(2*N)
+  p <- colMeans(M)/2
   
   if(any(p == 0 | p == 1))
-    stop("Monomorphic markers are no accepted")
+    stop("Monomorphic markers are no allowed")
 
   WWG <- function(M, p){
     w <- scale(x = M, center = T, scale = F)
@@ -72,33 +67,58 @@ G.matrix <- function(M, method=c("VanRaden", "UAR", "UARadj", "GK"), format=c("w
     return(g)
   }
   
-  if (method == "VanRaden"){
-    Gmat <- WWG(M, p)
-    namesG <- names(Gmat)
-    if (format == "long"){
-      Gmat <- lapply(Gmat, function(x) toSparse(posdefmat(x)))
-      
+   switch(method,
+         "VanRaden" = {
+           Ga <- WWG(M, p)
+           },
+         "UAR" = {
+           Ga <- UAR(M, p, metho = method)
+         },
+         "UARadj" = {
+           Ga <- UAR(M, p, metho = method)
+         },
+         "GK" = {
+           w <- scale(x = M, center = T, scale = T)
+           D <- as.matrix(dist(w)) ^ 2
+           if(quantile(D, 0.5) == 0)
+             stop("Was not possible to compute the 5% quantile for the distance matrix")
+           Ga <- exp(-D / quantile(D, 0.05))
+         },
+         {
+           stop("Method selected is not available")
+         })
+if(format == "long"){
+    if(is.list(Ga)){
+      G <- lapply(Ga, function(x) toSparse(posdefmat(x)))
+      names(G) <- names(Ga)
+    }else{
+      G <- toSparse(posdefmat(Ga))
     }
-    return(Gmat)
+    return(G)
+  }else{
+    if(plot){
+      if(method == "VanRaden"){
+        for(i in 1:length(Ga)){
+          pdf = pdf(paste("heatmap_", method, names(Ga)[i],".pdf", sep = ""), width = 10, height = 4)
+          heatmap(Ga[[i]], scale = "none", Rowv = NA, Colv = NA, cexRow = 0.5, 
+                  cexCol = 0.5,  main=paste(method, names(Ga)[i], sep = " "))
+          dev.off()
+        }
+      }else{
+        pdf = pdf(paste("heatmap_", method, ".pdf", sep = ""), width = 10, height = 4)
+        heatmap(Ga, scale = "none", Rowv = NA, Colv = NA, cexRow = 0.5, 
+                cexCol = 0.5,  main=method)
+        dev.off()		  
+      }
+      
+      tmp <- Ga
+      if(method == "VanRaden")
+        tmp <- tmp$Ga
+							        
+      pckin <- princomp(tmp, cor=TRUE, scores=TRUE)
+      #' @importFrom rgl plot3d
+      plot3d(pckin$scores[,1:3])
+    }
+    return(Ga)
   }
-  
-  if (method %in% c("UAR", "UARadj")){
-    uar <- UAR(M, p, metho = method)
-    if (format == "long")
-      {uar <- toSparse(posdefmat(uar))}
-    return(Ga=uar)
-  }
-  
-  if(method == "GK"){
-    w <- scale(x = M, center = T, scale = T)
-    D <- as.matrix(dist(w)) ^ 2
-    if(quantile(D, 0.5) == 0)
-      stop("Was not possible to compute the 50% quantile for the distance matrix")
-    GK <- exp(-D / quantile(D, 0.05))
-    
-    if(format == "long")
-      {GK <- toSparse(posdefmat(GK))}
-    return(GK)
-  }
-
-}
+}
\ No newline at end of file
